From bb7ecddaa351a363ab0086e2aef6a342e916a986 Mon Sep 17 00:00:00 2001
From: =?UTF-8?q?Hugo=20Beauz=C3=A9e-Luyssen?= <beauze.h@gmail.com>
Date: Wed, 15 Feb 2012 22:17:11 +0100
Subject: [PATCH 03/12] bluray: Preparing menu handling.

---
 modules/access/bluray.c |   72 +++++++++++++++++++++++++++++++++++++++-------
 1 files changed, 61 insertions(+), 11 deletions(-)

diff --git a/modules/access/bluray.c b/modules/access/bluray.c
index 2995773..acac745 100644
--- a/modules/access/bluray.c
+++ b/modules/access/bluray.c
@@ -44,6 +44,10 @@
  * Module descriptor
  *****************************************************************************/
 
+#define BD_MENU_TEXT        N_( "Bluray menus" )
+#define BD_MENU_LONGTEXT    N_( "Use bluray menus. If disabled, "\
+                                "the movie will start directly" )
+
 /* Callbacks */
 static int  blurayOpen ( vlc_object_t * );
 static void blurayClose( vlc_object_t * );
@@ -55,6 +59,7 @@ vlc_module_begin ()
     set_category( CAT_INPUT )
     set_subcategory( SUBCAT_INPUT_ACCESS )
     set_capability( "access_demux", 200)
+    add_bool( "bluray-menu", false, BD_MENU_TEXT, BD_MENU_LONGTEXT, false )
 
     add_shortcut( "bluray", "file" )
 
@@ -79,7 +84,8 @@ struct demux_sys_t
  * Local prototypes
  *****************************************************************************/
 static int     blurayControl(demux_t *, int, va_list);
-static int     blurayDemux  (demux_t *);
+static int     blurayDemux(demux_t *);
+static int     blurayDemuxMenu(demux_t *);
 
 static int     blurayInitTitles(demux_t *p_demux );
 static int     bluraySetTitle(demux_t *p_demux, int i_title);
@@ -198,17 +204,29 @@ static int blurayOpen( vlc_object_t *object )
      */
     bd_get_event( p_sys->bluray, NULL );
 
-    /* get title request */
-    if ((pos_title = strrchr(bd_path, ':'))) {
-        /* found character ':' for title information */
-        *(pos_title++) = '\0';
-        i_title = atoi(pos_title);
+    bool b_menu = var_InheritBool( p_demux, "bluray-menu" );
+    if ( b_menu )
+    {
+        /*
+         * libbluray will start playback from "First-Title" title
+         * Therefore, We don't have to select any title.
+         */
+        bd_play( p_sys->bluray );
     }
+    else
+    {
+        /* get title request */
+        if ((pos_title = strrchr(bd_path, ':'))) {
+            /* found character ':' for title information */
+            *(pos_title++) = '\0';
+            i_title = atoi(pos_title);
+        }
 
-    /* set start title number */
-    if (bluraySetTitle(p_demux, i_title) != VLC_SUCCESS) {
-        msg_Err( p_demux, "Could not set the title %d", i_title );
-        goto error;
+        /* set start title number */
+        if (bluraySetTitle(p_demux, i_title) != VLC_SUCCESS) {
+            msg_Err( p_demux, "Could not set the title %d", i_title );
+            goto error;
+        }
     }
 
     p_sys->p_parser   = stream_DemuxNew(p_demux, "ts", p_demux->out);
@@ -218,7 +236,7 @@ static int blurayOpen( vlc_object_t *object )
     }
 
     p_demux->pf_control = blurayControl;
-    p_demux->pf_demux   = blurayDemux;
+    p_demux->pf_demux   = b_menu ? blurayDemuxMenu : blurayDemux;
 
     return VLC_SUCCESS;
 
@@ -504,6 +522,7 @@ static int blurayDemux(demux_t *p_demux)
         return -1;
     }
 
+    //Handle events first.
     blurayHandleEvents( p_demux );
     int nread = bd_read(p_sys->bluray, p_block->p_buffer,
                         NB_TS_PACKETS * BD_TS_PACKET_SIZE);
@@ -515,6 +534,37 @@ static int blurayDemux(demux_t *p_demux)
     p_block->i_buffer = nread;
 
     stream_DemuxSend( p_sys->p_parser, p_block );
+    return 1;
+}
+
+static int blurayDemuxMenu( demux_t *p_demux )
+{
+    demux_sys_t *p_sys = p_demux->p_sys;
+    BD_EVENT    e;
 
+    block_t     *p_block = block_New(p_demux, NB_TS_PACKETS * (int64_t)BD_TS_PACKET_SIZE);
+    int         nread = bd_read_ext( p_sys->bluray, p_block->p_buffer,
+                                      NB_TS_PACKETS * BD_TS_PACKET_SIZE, &e );
+    /*
+     * bd_read_ext will return 0 if there is some events to process.
+     * We need to handle them before doing anything else
+     */
+    if ( nread == 0 )
+    {
+        if ( e.event == BD_EVENT_NONE )
+            msg_Info( p_demux, "We reached the end of a title" );
+        else
+            blurayHandleEvent( p_demux, &e );
+        block_Release(p_block);
+        return 1;
+    }
+    if (nread < 0)
+    {
+        block_Release(p_block);
+        return nread;
+    }
+    p_block->i_buffer = nread;
+
+    stream_DemuxSend( p_sys->p_parser, p_block );
     return 1;
 }
-- 
1.7.9.1

